#include "MAR_pp.def"
subroutine DYNadv_verq
    ! +------------------------------------------------------------------------+
    ! | MAR DYNAMICS   SLOW                                    18-09-2001  MAR |
    ! |   subroutine DYNadv_verq includes the Vertical  Advection Contribution |
    ! |                                                                        |
    ! +------------------------------------------------------------------------+
    ! |                                                                        |
    ! |   INPUT   (via common block)                                           |
    ! |   ^^^^^      iterun,            Run Iteration  Counter                 |
    ! |                qvDY,            Air Vap. Water Values / Time Step n    |
    ! |              qvapSL,            SBC Vap. Water Values / Time Step n    |
    ! |                                                                        |
    ! |   OUTPUT  (via common block)                                           |
    ! |   ^^^^^^       qvDY,            Air Vap. Water Values / Time Step n+1  |
    ! |                                                                        |
    ! |   METHOD:  Unstaggered Grid: 1st Accurate in Space Upstream Scheme     |
    ! |   ^^^^^^^^   Staggered Grid: 2nd Accurate in Space                     |
    ! |                                                                        |
    ! | # OPTIONS: #VA: Vertical Average preferred in Centered Conserv Scheme  |
    ! | # ^^^^^^^^ #NS: NO   Slip Surface BC used  in Centered Conserv Scheme  |
    ! |                                                                        |
    ! +------------------------------------------------------------------------+
    ! +
    use marctr
    use marphy
    use mardim
    use margrd
    use mar_dy
    use mar_sl
    use mar_wk
#if(cA)
    use mar_ca
#endif
    ! +
    implicit none
    ! +
#if(WA)
    integer nadvrd
    common / DYNadv_ver_loc / nadvrd
    ! +
#endif
    logical centrL
#if(ZU)
    logical adv3rd
    real gat(mx, my, mz), ga0(mx, my)
    data adv3rd/.true./
#endif
    ! +
    ! +
    ! +--Local  Variables
    ! +  ================
    ! +
    integer i, j, k, m
    integer itimax, ntimax
    real cflmax, cflsig, faccfl, dsgm, qv_0
    real old__u, old__v, old__t, qw_0, qr_0
    ! +
    ! +
    ! +--DATA
    ! +  ====
    ! +
    ! +
    data centrL/.true./
#if(UP)
    centrL = .false.
#endif
    ! +
    ! +
    ! +--Slip condition for Mountain Wave Experiments
    ! +  ============================================
    ! +
#if(OM)
    do j = jp11, my1
        do i = ip11, mx1
            psigDY(i, j, mz) = 0.0
        enddo
    enddo
#endif
    ! +
    ! +
    ! +--First and Second Order Schemes
    ! +  ==============================
    ! +
#if(ZU)
    if(.not. adv3rd) then
#endif
        ! +
        ! +
        ! +--Courant Number
        ! +  --------------
        ! +
        cflmax = 0.0
        ! +
        ! +--Centered second Order Scheme on a       staggered Grid
        ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if(centrL .and. staggr) then
            ! +
#if(WA)
            write(6, 6001) iterun
6001        format(i6, ' 6001       centrL .and.       staggr /CFL Number')
#endif
            ! +
            do k = 1, mz
                do j = jp11, my1
                    do i = ip11, mx1
                        WKxyz7(i, j, k) = dt * psigDY(i, j, k) &
                                          / (pstDYn(i, j) * dsigm1(k) * 2.0)
                        cflsig = abs(WKxyz7(i, j, k) + WKxyz7(i, j, k))
                        cflmax = max(cflsig, cflmax)
                    enddo
                enddo
            enddo
            ! +
            do j = jp11, my1
                do i = ip11, mx1
                    WKxyz8(i, j, 1) = 0.0
                enddo
            enddo
            ! +
            do k = kp1(1), mz
                do j = jp11, my1
                    do i = ip11, mx1
                        WKxyz8(i, j, k) = dt * psigDY(i, j, km1(k)) &
                                          / (pstDYn(i, j) * dsigm1(k) * 2.0)
                        cflsig = abs(WKxyz8(i, j, k) + WKxyz8(i, j, k))
                        cflmax = max(cflsig, cflmax)
                    enddo
                enddo
            enddo
            ! +
        else
            ! +
            ! +--Upstream first  Order Scheme on a       staggered Grid
            ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            if(staggr) then
                ! +
#if(WA)
                write(6, 6002) iterun
6002            format(i6, ' 6002 .not. centrL .and.       staggr /Wind Speed')
#endif
                ! +
                do k = kp1(1), mz
                    do j = jp11, my1
                        do i = ip11, mx1
                            WKxyz8(i, j, k) = (psigDY(i, j, k - 1) * dsig_1(k - 1) &
                                               + psigDY(i, j, k) * dsig_1(k)) &
                                              / (dsig_1(k - 1) + dsig_1(k))
                        enddo
                    enddo
                enddo
                ! +
                do j = jp11, my1
                    do i = ip11, mx1
                        WKxyz8(i, j, 1) = psigDY(i, j, 1) * dsig_1(1) &
                                          / (dsig_1(0) + dsig_1(1))
                    enddo
                enddo
                ! +
                ! +--Upstream first  Order Scheme on a  non  staggered Grid
                ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            else
                ! +
#if(WA)
                write(6, 6003) iterun
6003            format(i6, ' 6003 (.not.)centrL.and. .not. staggr /Wind Speed')
#endif
                ! +
                do k = 1, mz
                    do j = jp11, my1
                        do i = ip11, mx1
                            WKxyz8(i, j, k) = psigDY(i, j, k)
                        enddo
                    enddo
                enddo
                ! +
            endif
            ! +
            ! +--Centered second Order Scheme on a  non  staggered Grid
            ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            if(centrL) then
                ! +
#if(WA)
                write(6, 6004) iterun
6004            format(i6, ' 6004        centrL.and. .not. staggr /CFL Number')
#endif
                ! +
                do k = 1, mz
                    do j = jp11, my1
                        do i = ip11, mx1
                            WKxyz7(i, j, k) = dt * WKxyz8(i, j, k) &
                                              / (pstDYn(i, j) * dsigm1(k) * 2.0)
                            cflsig = abs(WKxyz7(i, j, k))
                            cflmax = max(cflsig, cflmax)
                        enddo
                    enddo
                enddo
                ! +
                ! +--Upstream first  Order Scheme on a (non) staggered Grid
                ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            else
                ! +
#if(WA)
                write(6, 6005) iterun
6005            format(i6, ' 6005  .not. centrL.and.(.not.)staggr /CFL Number')
#endif
                ! +
                do k = 1, mmz1
                    do j = jp11, my1
                        do i = ip11, mx1
                            if(WKxyz8(i, j, k) > 0.0) then
                                WKxyz7(i, j, k) = -dt * WKxyz8(i, j, k) / (pstDYn(i, j) * dsig_1(k - 1))
                            else
                                WKxyz7(i, j, k) = -dt * WKxyz8(i, j, k) / (pstDYn(i, j) * dsig_1(k))
                            endif
                            cflsig = abs(WKxyz7(i, j, k))
                            cflmax = max(cflsig, cflmax)
                        enddo
                    enddo
                enddo
                ! +
                k = mz
                do j = jp11, my1
                    do i = ip11, mx1
                        if(WKxyz8(i, j, k) > 0.0) then
                            WKxyz7(i, j, k) = -dt * WKxyz8(i, j, k) / (pstDYn(i, j) * dsig_1(k - 1))
                        else
                            WKxyz7(i, j, k) = -dt * WKxyz8(i, j, k) / (pstDYn(i, j) * dsig_1(k))
                        endif
                        cflsig = abs(WKxyz7(i, j, k))
                        cflmax = max(cflsig, cflmax)
                    enddo
                enddo
                ! +
                do j = 1, my
                    do i = 1, mx
                        WKxyz7(i, j, 1) = 0.0
                    enddo
                enddo
                ! +
                ! +--Work Array Reset
                ! +  ~~~~~~~~~~~~~~~~
                do k = 1, mz
                    do j = 1, my
                        do i = 1, mx
                            WKxyz8(i, j, k) = 0.0
                        enddo
                    enddo
                enddo
                ! +
            endif
            ! +
        endif
        ! +
        ! +
        ! +--Set Up    of the Local Split Time Differencing
        ! +  ----------------------------------------------
        ! +
        cflmax = 2.0 * cflmax
        ! +...      restricted CFL Criterion
        ! +
        ntimax = cflmax
        if(centrL) then
            ntimax = max(2, ntimax)
#if(WA)
            write(6, 6006) ntimax
6006        format(i6, ' 6006        centrL.and.(.not.)staggr /Nb Iterat.')
#endif
        else
            ntimax = max(1, ntimax)
#if(WA)
            write(6, 6007) ntimax
6007        format(i6, ' 6007  .not. centrL.and.(.not.)staggr /Nb Iterat.')
#endif
        endif
        ! +
        ! +--Update of CFL Number
        ! +  ~~~~~~~~~~~~~~~~~~~~
        if(ntimax > 1) then
            faccfl = 1.0d+0 / ntimax
            do k = 1, mz
                do j = jp11, my1
                    do i = ip11, mx1
                        WKxyz7(i, j, k) = WKxyz7(i, j, k) * faccfl
                        WKxyz8(i, j, k) = WKxyz8(i, j, k) * faccfl
                    enddo
                enddo
            enddo
        endif
        ! +
        ! +--OUTPUT for Verification
        ! +  ~~~~~~~~~~~~~~~~~~~~~~~
#if(WA)
        nadvrd = nadvrd + 1
        write(6, 6000) nadvrd, cflmax, ntimax
6000    format(i6, ' CFLmax ', 3x, ' ', 3x, '  =', f7.4, &
               6x, ' ntimax ', 8x, '   =', i4)
#endif
        ! +
        ! +
        ! +--2nd Order Centered Energy conserving:  Local Split Time Differencing
        ! +  --------- (Haltiner & Williams 1980 7.2.2, (7-47b) p.220) ----------
        ! +            -----------------------------------------------
        ! +
        if(centrL) then
            ! +
            if(staggr) then
                ! +
#if(WA)
                write(6, 6008)
6008            format(6x, ' 6008        centrL.and.       staggr /A Contrib.')
#endif
                ! +
                do itimax = 1, ntimax
                    ! +
                    ! +--First        internal Time Step
                    ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    if(itimax == 1) then
                        ! +
                        do j = jp11, my1
                            ! +
                            ! +--Vertical Differences
                            ! +
                            k = 1
                            dsgm = 2.0 * dsigm1(k) + dsigm1(k + 1)
                            do i = ip11, mx1
                                WKxzx(i, k) = qvDY(i, j, k)
                                ! +
#if(VA)
                                WKxzx(i, k) = (qvDY(i, j, k) * dsigm1(k) * 2.0 &
                                               + qvDY(i, j, k + 1) * dsigm1(k + 1)) / dsgm
#endif
                            enddo
                            ! +
                            do i = ip11, mx1
                                qv_0 = WKxzx(i, k)
                                WKxzc(i, k) = (WKxzx(i, k) - qv_0)
                            enddo
                            ! +
                            do k = kp1(1), mmz1
                                dsgm = dsigm1(k - 1) + 2.0 * dsigm1(k) + dsigm1(k + 1)
                                do i = ip11, mx1
                                    WKxzx(i, k) = qvDY(i, j, k)
                                    ! +
#if(VA)
                                    WKxzx(i, k) = (qvDY(i, j, k - 1) * dsigm1(k - 1) &
                                                   + qvDY(i, j, k) * dsigm1(k) * 2.0d+0 &
                                                   + qvDY(i, j, k + 1) * dsigm1(k + 1)) / dsgm
#endif
                                enddo
                            enddo
                            ! +
                            do k = kp1(1), mmz1
                                do i = ip11, mx1
                                    WKxzc(i, k) = (WKxzx(i, k) - WKxzx(i, k - 1))
                                enddo
                            enddo
                            ! +
                            k = mmz
                            dsgm = dsigm1(k - 1) + 2.0 * dsigm1(k)
                            do i = ip11, mx1
                                WKxzx(i, k) = qvDY(i, j, k)
                                ! +
#if(VA)
                                WKxzx(i, k) = (qvDY(i, j, k - 1) * dsigm1(k - 1) &
                                               + qvDY(i, j, k) * dsigm1(k) * 2.0) / dsgm
#endif
                            enddo
                            ! +
                            do i = ip11, mx1
                                WKxzc(i, k) = (WKxzx(i, k) - WKxzx(i, k - 1))
                            enddo
                            ! +
                            k = mzz
                            do i = ip11, mx1
                                WKxy3(i, j) = 0.0
#if(NS)
                                WKxy3(i, j) = (qvapSL(i, j) - WKxzx(i, k - 1))
#endif
                            enddo
                            ! +
                            ! +--Advection Contribution
                            ! +
                            do k = 1, mmz1
                                do i = ip11, mx1
                                    WKxzd(i, k) = WKxyz7(i, j, k) * WKxzc(i, k + 1) &
                                                  + WKxyz8(i, j, k) * WKxzc(i, k)
                                    WKxyz3(i, j, k) = qvDY(i, j, k) - WKxzd(i, k)
                                    WKxyz6(i, j, k) = qvDY(i, j, k) - (WKxzd(i, k) + WKxzd(i, k))
                                enddo
                            enddo
                            ! +
                            k = mmz
                            do i = ip11, mx1
                                WKxzd(i, k) = WKxyz7(i, j, k) * WKxy3(i, j) &
                                              + WKxyz8(i, j, k) * WKxzc(i, k)
                                WKxyz3(i, j, k) = qvDY(i, j, k) - WKxzd(i, k)
                                WKxyz6(i, j, k) = qvDY(i, j, k) - (WKxzd(i, k) + WKxzd(i, k))
                            enddo
                            ! +
                        enddo
                        ! +
                        ! +--Intermediary internal Time Step
                        ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    else if(itimax < ntimax) then
                        ! +
                        ! +--Vertical Differences
                        ! +
                        do j = jp11, my1
                            ! +
                            k = 1
                            dsgm = 2.0 * dsigm1(k) + dsigm1(k + 1)
                            do i = ip11, mx1
                                WKxzx(i, k) = WKxyz6(i, j, k)
                                ! +
#if(VA)
                                WKxzx(i, k) = (WKxyz6(i, j, k) * dsigm1(k) * 2.0 &
                                               + WKxyz6(i, j, k + 1) * dsigm1(k + 1)) / dsgm
#endif
                            enddo
                            ! +
                            do i = ip11, mx1
                                qv_0 = WKxzx(i, k)
                                WKxzc(i, k) = (WKxzx(i, k) - qv_0)
                            enddo
                            ! +
                            do k = kp1(1), mmz1
                                dsgm = dsigm1(k - 1) + 2.0 * dsigm1(k) + dsigm1(k + 1)
                                do i = ip11, mx1
                                    WKxzx(i, k) = WKxyz6(i, j, k)
                                    ! +
#if(VA)
                                    WKxzx(i, k) = (WKxyz6(i, j, k - 1) * dsigm1(k - 1) &
                                                   + WKxyz6(i, j, k) * dsigm1(k) * 2.0d+0 &
                                                   + WKxyz6(i, j, k + 1) * dsigm1(k + 1)) / dsgm
#endif
                                enddo
                            enddo
                            ! +
                            do k = kp1(1), mmz1
                                do i = ip11, mx1
                                    WKxzc(i, k) = (WKxzx(i, k) - WKxzx(i, k - 1))
                                enddo
                            enddo
                            ! +
                            k = mmz
                            dsgm = dsigm1(k - 1) + 2.0 * dsigm1(k)
                            do i = ip11, mx1
                                WKxzx(i, k) = WKxyz6(i, j, k)
                                ! +
#if(VA)
                                WKxzx(i, k) = (WKxyz6(i, j, k - 1) * dsigm1(k - 1) &
                                               + WKxyz6(i, j, k) * dsigm1(k) * 2.0) / dsgm
#endif
                            enddo
                            ! +
                            do i = ip11, mx1
                                WKxzc(i, k) = (WKxzx(i, k) - WKxzx(i, k - 1))
                            enddo
                            ! +
                            k = mzz
                            do i = ip11, mx1
                                WKxy3(i, j) = 0.0d+0
#if(NS)
                                WKxy3(i, j) = (qvapSL(i, j) - WKxzx(i, k - 1))
#endif
                            enddo
                            ! +
                            ! +--Advection Contribution
                            ! +
                            do k = 1, mmz1
                                do i = ip11, mx1
                                    WKxzd(i, k) = WKxyz7(i, j, k) * WKxza(i, k + 1) &
                                                  + WKxyz8(i, j, k) * WKxza(i, k)
                                    old__u = WKxyz1(i, j, k)
                                    WKxyz1(i, j, k) = WKxyz4(i, j, k)
                                    WKxyz4(i, j, k) = old__u - (WKxzd(i, k) + WKxzd(i, k))
                                    ! +
                                    WKxzd(i, k) = WKxyz7(i, j, k) * WKxzb(i, k + 1) &
                                                  + WKxyz8(i, j, k) * WKxzb(i, k)
                                    old__v = WKxyz2(i, j, k)
                                    WKxyz2(i, j, k) = WKxyz5(i, j, k)
                                    WKxyz5(i, j, k) = old__v - (WKxzd(i, k) + WKxzd(i, k))
                                    ! +
                                    WKxzd(i, k) = WKxyz7(i, j, k) * WKxzc(i, k + 1) &
                                                  + WKxyz8(i, j, k) * WKxzc(i, k)
                                    old__t = WKxyz3(i, j, k)
                                    WKxyz3(i, j, k) = WKxyz6(i, j, k)
                                    WKxyz6(i, j, k) = old__t - (WKxzd(i, k) + WKxzd(i, k))
                                enddo
                            enddo
                            ! +
                            k = mmz
                            do i = ip11, mx1
                                WKxzd(i, k) = WKxyz7(i, j, k) * WKxy1(i, j) &
                                              + WKxyz8(i, j, k) * WKxza(i, k)
                                old__u = WKxyz1(i, j, k)
                                WKxyz1(i, j, k) = WKxyz4(i, j, k)
                                WKxyz4(i, j, k) = old__u - (WKxzd(i, k) + WKxzd(i, k))
                                ! +
                                WKxzd(i, k) = WKxyz7(i, j, k) * WKxy2(i, j) &
                                              + WKxyz8(i, j, k) * WKxzb(i, k)
                                old__v = WKxyz2(i, j, k)
                                WKxyz2(i, j, k) = WKxyz5(i, j, k)
                                WKxyz5(i, j, k) = old__v - (WKxzd(i, k) + WKxzd(i, k))
                                ! +
                                WKxzd(i, k) = WKxyz7(i, j, k) * WKxy3(i, j) &
                                              + WKxyz8(i, j, k) * WKxzc(i, k)
                                old__t = WKxyz3(i, j, k)
                                WKxyz3(i, j, k) = WKxyz6(i, j, k)
                                WKxyz6(i, j, k) = old__t - (WKxzd(i, k) + WKxzd(i, k))
                            enddo
                            ! +
                        enddo
                        ! +
                        ! +--Last         internal Time Step
                        ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    else
                        ! +
                        do j = jp11, my1
                            ! +
                            ! +--Vertical Differences
                            ! +
                            k = 1
                            dsgm = 2.0 * dsigm1(k) + dsigm1(k + 1)
                            do i = ip11, mx1
                                WKxzx(i, k) = WKxyz6(i, j, k)
                                ! +
#if(VA)
                                WKxzx(i, k) = (WKxyz6(i, j, k) * dsigm1(k) * 2.0 &
                                               + WKxyz6(i, j, k + 1) * dsigm1(k + 1)) / dsgm
#endif
                            enddo
                            ! +
                            do i = ip11, mx1
                                qv_0 = WKxzx(i, k)
                                WKxzc(i, k) = (WKxzx(i, k) - qv_0)
                            enddo
                            ! +
                            do k = kp1(1), mmz1
                                dsgm = dsigm1(k - 1) + 2.0 * dsigm1(k) + dsigm1(k + 1)
                                do i = ip11, mx1
                                    WKxzx(i, k) = WKxyz6(i, j, k)
                                    ! +
#if(VA)
                                    WKxzx(i, k) = (WKxyz6(i, j, k - 1) * dsigm1(k - 1) &
                                                   + WKxyz6(i, j, k) * dsigm1(k) * 2.0 &
                                                   + WKxyz6(i, j, k + 1) * dsigm1(k + 1)) / dsgm
#endif
                                enddo
                            enddo
                            ! +
                            do k = kp1(1), mmz1
                                do i = ip11, mx1
                                    WKxzc(i, k) = (WKxzx(i, k) - WKxzx(i, k - 1))
                                enddo
                            enddo
                            ! +
                            k = mmz
                            dsgm = dsigm1(k - 1) + 2.0 * dsigm1(k)
                            do i = ip11, mx1
                                WKxzx(i, k) = WKxyz6(i, j, k)
                                ! +
#if(VA)
                                WKxzx(i, k) = (WKxyz6(i, j, k - 1) * dsigm1(k - 1) &
                                               + WKxyz6(i, j, k) * dsigm1(k) * 2.0) / dsgm
#endif
                            enddo
                            ! +
                            do i = ip11, mx1
                                WKxzc(i, k) = (WKxzx(i, k) - WKxzx(i, k - 1))
                            enddo
                            ! +
                            k = mzz
                            do i = ip11, mx1
                                WKxy3(i, j) = 0.0d+0
#if(NS)
                                WKxy3(i, j) = (qvapSL(i, j) - WKxzx(i, k - 1))
#endif
                            enddo
                            ! +
                            ! +--Wat.Vapr.Advect.avoids double Counting in case of convective Adjustment
                            ! +
                            do k = 1, mmz1
                                do i = ip11, mx1
#if(cA)
                                    if(adj_CA(i, j) == 0) then
#endif
                                        qvDY(i, j, k) = WKxyz3(i, j, k) &
                                                        - (WKxyz7(i, j, k) * WKxzc(i, k + 1) &
                                                           + WKxyz8(i, j, k) * WKxzc(i, k))
#if(cA)
                                    endif
#endif
                                enddo
                            enddo
                            ! +
                            k = mmz
                            do i = ip11, mx1
#if(cA)
                                if(adj_CA(i, j) == 0) then
#endif
                                    qvDY(i, j, k) = WKxyz3(i, j, k) &
                                                    - (WKxyz7(i, j, k) * WKxy3(i, j) &
                                                       + WKxyz8(i, j, k) * WKxzc(i, k))
#if(cA)
                                endif
#endif
                            enddo
                            ! +
                        enddo
                        ! +
                    endif
                    ! +
                    ! +
                    ! +--End of the                             Local Split Time Differencing
                    ! +  --------------------------------------------------------------------
                    ! +
                enddo
                ! +
                ! +
                ! +--2nd Order Centered Leap-Frog Backward: Local Split Time Differencing
                ! +  --------------------------------------------------------------------
                ! +
            else
                ! +
#if(WA)
                write(6, 6009)
6009            format(6x, ' 6009        centrL.and. .not. staggr /A Contrib.')
#endif
                ! +
                do itimax = 1, ntimax
                    ! +
                    ! +--First        internal Time Step
                    ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    if(itimax == 1) then
                        ! +
                        do j = jp11, my1
                            ! +
                            ! +--Advection Increment
                            ! +
                            k = 1
                            do i = ip11, mx1
                                qv_0 = qvDY(i, j, k)
                                WKxzc(i, k) = (qvDY(i, j, k + 1) - qvDY(i, j, k)) &
                                              * WKxyz7(i, j, k)
                            enddo
                            ! +
                            do k = kp1(1), mmz1
                                do i = ip11, mx1
                                    WKxzc(i, k) = (qvDY(i, j, k + 1) - qvDY(i, j, k - 1)) &
                                                  * WKxyz7(i, j, k)
                                enddo
                            enddo
                            ! +
                            k = mmz
                            do i = ip11, mx1
                                WKxzc(i, k) = (qvapSL(i, j) - qvDY(i, j, k - 1)) &
                                              * WKxyz7(i, j, k)
                            enddo
                            ! +
                            ! +--Advection Contribution
                            ! +
                            do k = 1, mmz
                                do i = ip11, mx1
                                    WKxyz3(i, j, k) = qvDY(i, j, k) - WKxzc(i, k)
                                    WKxyz6(i, j, k) = qvDY(i, j, k) - (WKxzc(i, k) + WKxzc(i, k))
                                enddo
                            enddo
                        enddo
                        ! +
                        ! +--Intermediary internal Time Step
                        ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    else if(itimax < ntimax) then
                        ! +
                        ! +--Advection Increment
                        ! +
                        do j = jp11, my1
                            ! +
                            k = 1
                            do i = ip11, mx1
                                qw_0 = WKxyz4(i, j, k)
                                qr_0 = WKxyz5(i, j, k)
                                qv_0 = WKxyz6(i, j, k)
                                ! +
                                WKxza(i, k) = (WKxyz4(i, j, k + 1) - qw_0) &
                                              * WKxyz7(i, j, k)
                                WKxzb(i, k) = (WKxyz5(i, j, k + 1) - qr_0) &
                                              * WKxyz7(i, j, k)
                                WKxzc(i, k) = (WKxyz6(i, j, k + 1) - qv_0) &
                                              * WKxyz7(i, j, k)
                            enddo
                            ! +
                            do k = kp1(1), mmz1
                                do i = ip11, mx1
                                    WKxza(i, k) = (WKxyz4(i, j, k + 1) - WKxyz4(i, j, k - 1)) &
                                                  * WKxyz7(i, j, k)
                                    WKxzb(i, k) = (WKxyz5(i, j, k + 1) - WKxyz5(i, j, k - 1)) &
                                                  * WKxyz7(i, j, k)
                                    WKxzc(i, k) = (WKxyz6(i, j, k + 1) - WKxyz6(i, j, k - 1)) &
                                                  * WKxyz7(i, j, k)
                                enddo
                            enddo
                            ! +
                            k = mmz
                            do i = ip11, mx1
                                WKxza(i, k) = -WKxyz4(i, j, k - 1) &
                                              * WKxyz7(i, j, k)
                                WKxzb(i, k) = -WKxyz5(i, j, k - 1) &
                                              * WKxyz7(i, j, k)
                                WKxzc(i, k) = (qvapSL(i, j) - WKxyz6(i, j, k - 1)) &
                                              * WKxyz7(i, j, k)
                            enddo
                            ! +
                            ! +--Advection Contribution
                            ! +
                            do k = 1, mmz
                                do i = ip11, mx1
                                    old__u = WKxyz1(i, j, k)
                                    WKxyz1(i, j, k) = WKxyz4(i, j, k)
                                    WKxyz4(i, j, k) = old__u - (WKxza(i, k) + WKxza(i, k))
                                    old__v = WKxyz2(i, j, k)
                                    WKxyz2(i, j, k) = WKxyz5(i, j, k)
                                    WKxyz5(i, j, k) = old__v - (WKxzb(i, k) + WKxzb(i, k))
                                    old__t = WKxyz3(i, j, k)
                                    WKxyz3(i, j, k) = WKxyz6(i, j, k)
                                    WKxyz6(i, j, k) = old__t - (WKxzc(i, k) + WKxzc(i, k))
                                enddo
                            enddo
                            ! +
                        enddo
                        ! +
                        ! +--Last         internal Time Step
                        ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    else
                        ! +
                        do j = jp11, my1
                            ! +
                            ! +--Advection Increment
                            ! +
                            k = 1
                            do i = ip11, mx1
                                qw_0 = WKxyz4(i, j, k)
                                qr_0 = WKxyz5(i, j, k)
                                qv_0 = WKxyz6(i, j, k)
                                ! +
                                WKxza(i, k) = (WKxyz4(i, j, k + 1) - qw_0) &
                                              * WKxyz7(i, j, k)
                                WKxzb(i, k) = (WKxyz5(i, j, k + 1) - qr_0) &
                                              * WKxyz7(i, j, k)
                                WKxzc(i, k) = (WKxyz6(i, j, k + 1) - qv_0) &
                                              * WKxyz7(i, j, k)
                            enddo
                            ! +
                            do k = kp1(1), mmz1
                                do i = ip11, mx1
                                    WKxza(i, k) = (WKxyz4(i, j, k + 1) - WKxyz4(i, j, k - 1)) &
                                                  * WKxyz7(i, j, k)
                                    WKxzb(i, k) = (WKxyz5(i, j, k + 1) - WKxyz5(i, j, k - 1)) &
                                                  * WKxyz7(i, j, k)
                                    WKxzc(i, k) = (WKxyz6(i, j, k + 1) - WKxyz6(i, j, k - 1)) &
                                                  * WKxyz7(i, j, k)
                                enddo
                            enddo
                            ! +
                            k = mmz
                            do i = ip11, mx1
                                WKxza(i, k) = -WKxyz4(i, j, k - 1) &
                                              * WKxyz7(i, j, k)
                                WKxzb(i, k) = -WKxyz5(i, j, k - 1) &
                                              * WKxyz7(i, j, k)
                                WKxzc(i, k) = (qvapSL(i, j) - WKxyz6(i, j, k - 1)) &
                                              * WKxyz7(i, j, k)
                            enddo
                            ! +
                            ! +--Wat.Vapr.Advect.avoids double Counting in case of convective Adjustment
                            ! +
                            do k = 1, mmz
                                do i = ip11, mx1
#if(cA)
                                    if(adj_CA(i, j) == 0) then
#endif
                                        qvDY(i, j, k) = WKxyz3(i, j, k) - WKxzc(i, k)
#if(cA)
                                    endif
#endif
                                enddo
                            enddo
                            ! +
                        enddo
                        ! +
                    endif
                    ! +
                    ! +
                    ! +--End of the                             Local Split Time Differencing
                    ! +  --------------------------------------------------------------------
                    ! +
                enddo
                ! +
            endif
            ! +
            ! +
            ! +--First  Order Upstream Scheme:          Local Split Time Differencing
            ! +  --------------------------------------------------------------------
            ! +
        else
            ! +
#if(WA)
            write(6, 6010)
6010        format(6x, ' 6010  .not. centrL.and.(.not.)staggr /A Contrib.')
#endif
            ! +
            do itimax = 1, ntimax
                ! +
                ! +--Auxiliary Variables
                ! +  ~~~~~~~~~~~~~~~~~~~
#if(WA)
                write(6, 6011) itimax, WKxyz1(imez, jmez, mz1), WKxyz1(imez, jmez, mz) &
                    , qvDY(imez, jmez, mz1), qvDY(imez, jmez, mz)
#endif
6011            format(6x, ' 6011  .not. centrL.and.(.not.)staggr /A Contrib.', &
                       4f9.6)
                ! +
                do k = 1, mz
                    do j = jp11, my1
                        do i = ip11, mx1
                            WKxyz3(i, j, k) = qvDY(i, j, k)
                        enddo
                    enddo
                enddo
                ! +
                ! +--Vertical Differences
                ! +  ~~~~~~~~~~~~~~~~~~~~
                k = 1
                do j = jp11, my1
                    do i = ip11, mx1
                        WKxyz4(i, j, k) = 0.0d+0
                        WKxyz5(i, j, k) = 0.0d+0
                        WKxyz6(i, j, k) = 0.0d+0
                    enddo
                enddo
                ! +
                do k = kp1(1), mz
                    do j = jp11, my1
                        do i = ip11, mx1
                            WKxyz4(i, j, k) = WKxyz1(i, j, k) - WKxyz1(i, j, k - 1)
                            WKxyz5(i, j, k) = WKxyz2(i, j, k) - WKxyz2(i, j, k - 1)
                            WKxyz6(i, j, k) = WKxyz3(i, j, k) - WKxyz3(i, j, k - 1)
                        enddo
                    enddo
                enddo
                ! +
                k = mzz
                do j = jp11, my1
                    do i = ip11, mx1
                        WKxy1(i, j) = -WKxyz1(i, j, k - 1)
                        WKxy2(i, j) = -WKxyz2(i, j, k - 1)
                        WKxy3(i, j) = qvapSL(i, j) - WKxyz3(i, j, k - 1)
                    enddo
                enddo
                ! +
                ! +--Advection Contribution
                ! +  ~~~~~~~~~~~~~~~~~~~~~~
                do k = 1, mmz1
                    do j = jp11, my1
                        do i = ip11, mx1
                            WKxyz3(i, j, k) = qvDY(i, j, k) &
                                              + min(zero, WKxyz7(i, j, k)) * WKxyz6(i, j, k) &
                                              + max(zero, WKxyz7(i, j, k)) * WKxyz6(i, j, k + 1)
                        enddo
                    enddo
                enddo
                ! +
                k = mz
                do j = jp11, my1
                    do i = ip11, mx1
                        WKxyz3(i, j, k) = qvDY(i, j, k) &
                                          + min(zero, WKxyz7(i, j, k)) * WKxyz6(i, j, k) &
                                          + max(zero, WKxyz7(i, j, k)) * WKxy3(i, j)
                    enddo
                enddo
                ! +
                ! +--Wat.Vapr.Update avoids double Counting in case of convective Adjustment
                ! +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                do k = 1, mz
                    do j = jp11, my1
                        do i = ip11, mx1
#if(cA)
                            if(adj_CA(i, j) == 0) then
#endif
                                qvDY(i, j, k) = WKxyz3(i, j, k)
#if(cA)
                            endif
#endif
                        enddo
                    enddo
                enddo
                ! +
                ! +
                ! +--End of the                             Local Split Time Differencing
                ! +  --------------------------------------------------------------------
                ! +
#if(WA)
                write(6, 6012) itimax, WKxyz1(imez, jmez, mz1), WKxyz1(imez, jmez, mz) &
                    , qvDY(imez, jmez, mz1), qvDY(imez, jmez, mz)
6012            format(6x, ' 6012  .not. centrL.and.(.not.)staggr /A Contrib.', &
                       4f9.6)
#endif
            enddo
            ! +
        endif
        ! +
        ! +
        ! +--Work Arrays Reset
        ! +  -----------------
        ! +
        do j = 1, my
            do i = 1, mx
                WKxy1(i, j) = 0.0
                WKxy2(i, j) = 0.0
                WKxy3(i, j) = 0.0
            enddo
        enddo
        ! +
        do k = 1, mz
            do i = 1, mx
                WKxza(i, k) = 0.0
                WKxzb(i, k) = 0.0
                WKxzc(i, k) = 0.0
                WKxzd(i, k) = 0.0
            enddo
        enddo
        ! +
        do k = 1, mz
            do j = 1, my
                do i = 1, mx
                    WKxyz1(i, j, k) = 0.0
                    WKxyz2(i, j, k) = 0.0
                    WKxyz3(i, j, k) = 0.0
                    WKxyz4(i, j, k) = 0.0
                    WKxyz5(i, j, k) = 0.0
                    WKxyz6(i, j, k) = 0.0
                    WKxyz7(i, j, k) = 0.0
                    WKxyz8(i, j, k) = 0.0
                enddo
            enddo
        enddo
        ! +
        ! +
        ! +--Third Order Vertical Scheme
        ! +  ===========================
        ! +
#if(ZU)
    else
        do j = jp11, my1
            do i = ip11, mx1
                ga0(i, j) = qvapSL(i, j)
#endif
#if(ZO)
                ga0(i, j) = qvDY(i, j, mz)
#endif
#if(ZU)
            enddo
        enddo
        do k = 1, mz
            do j = jp11, my1
                do i = ip11, mx1
                    gat(i, j, k) = qvDY(i, j, k)
                enddo
            enddo
        enddo
        ! +    ****************
        call DYNadv_cubv(gat, ga0)
        ! +    ****************
        do k = 1, mz
            do j = jp11, my1
                do i = ip11, mx1
                    qvDY(i, j, k) = gat(i, j, k)
                enddo
            enddo
        enddo
    endif
#endif
    return
endsubroutine DYNadv_verq
